/*
 * Copyright (c) 2008, Felix Opatz
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *  * Neither the name of  nor the names of its contributors may be used to
 *    endorse or promote products derived from this software without specific
 *    prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 *
 * Date        Change  
 * ---------------------------------------------------------------------------
 * 2008-10-28  Initial implementation.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <windows.h>

#include "zjlink.h"

/* Opens the COM port pcszName for both directions */
HPORT OpenComPort(PCSTR pcszName)
{
    return CreateFile(pcszName, GENERIC_READ | GENERIC_WRITE,
                      0, NULL, OPEN_EXISTING, 0, NULL);
}

/* Configure COM port hPort */
BOOL ConfigureComPort(HPORT hPort, DWORD dwBaudRate, PCSTR lpszModeString)
{
    DCB dcb;

    /* Mode string with != 3 characters is malformed */
    if (strlen(lpszModeString) != 3)
        return FALSE;

    ZeroMemory(&dcb, sizeof(dcb));
    dcb.DCBlength = sizeof(dcb);
    dcb.BaudRate = dwBaudRate;
    dcb.fBinary = TRUE;
    dcb.ByteSize = lpszModeString[0] - '0';

    switch (lpszModeString[1])
    {
        case 'n':
        case 'N':
            dcb.Parity = NOPARITY;
            break;
        case 'o':
        case 'O':
            dcb.Parity = ODDPARITY;
            break;
        case 'e':
        case 'E':
            dcb.Parity = EVENPARITY;
            break;
        default:
            return FALSE;
    }

    switch (lpszModeString[2])
    {
        case '1':
            dcb.StopBits = ONESTOPBIT;
            break;
        case '2':
            dcb.StopBits = TWOSTOPBITS;
            break;
        default:
            return FALSE;
    }

    if (SetCommState(hPort, &dcb) == FALSE)
    {
        fprintf(stderr, "SetCommState() failed with %lu\n", GetLastError());
        return FALSE;
    }

    return TRUE;
}

/* Read a byte */
BYTE ReadByte(HPORT hPort)
{
    BYTE bData;
    DWORD dwBytesRead;

    do
    {
        ReadFile(hPort, &bData, 1, &dwBytesRead, NULL);
    }
    while (dwBytesRead < 1);

    return bData;
}

/* Write a byte */
void WriteByte(HPORT hPort, BYTE bByte)
{
    DWORD dwBytesWritten;

    do
    {
        WriteFile(hPort, &bByte, 1, &dwBytesWritten, NULL);
    }
    while (dwBytesWritten < 1);
}

/* Close the COM port */
void CloseComPort(HPORT hPort)
{
    CloseHandle(hPort);
}
