/*
 * Copyright (c) 2008, Felix Opatz
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *  * Neither the name of  nor the names of its contributors may be used to
 *    endorse or promote products derived from this software without specific
 *    prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 *
 * Date        Change  
 * ---------------------------------------------------------------------------
 * 2008-10-28  Initial implementation.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <conio.h>

#include "uart.h"

#include "zjlink.h"

#define ESC_STROKES_TO_QUIT 3
#define ESC_KEYCODE         0x1b

/* Adaptions to the Borland Turbo C compiler. Microsoft C users may need to
 * change this into _outp and _inp.
 */
#define OUTB outport
#define INB  inport

/* "open" COM port, just return base address */
HPORT OpenComPort(PCSTR pcszName)
{
    if (!stricmp(pcszName, "COM1"))
        return UART_BASE_ADDR_COM1;

    if (!stricmp(pcszName, "COM2"))
        return UART_BASE_ADDR_COM2;

    if (!stricmp(pcszName, "COM3"))
        return UART_BASE_ADDR_COM3;

    if (!stricmp(pcszName, "COM4"))
        return UART_BASE_ADDR_COM4;

    return INVALID_PORT_HANDLE;
}

/* Configure COM port at base address hPort */
BOOL ConfigureComPort(HPORT hPort, DWORD dwBaudRate, PCSTR lpszModeString)
{
    BYTE bSettings = 0;
    DWORD dwBaudRateDivisor;

    /* Disable interrupts */
    OUTB(hPort + UART_OFFSET_IER, 0);

    /* Enable access to divisor latches */
    OUTB(hPort + UART_OFFSET_LCR, (1 << UART_LCR_DLAB));

    /* Calculate divisor for requested baudrate */
    dwBaudRateDivisor = UART_MAX_BAUDRATE / dwBaudRate;
    /* Set divisor latch low byte */
    OUTB(hPort + UART_OFFSET_DLL, dwBaudRateDivisor & 0xFF);
    /* Set divisor latch high byte */
    OUTB(hPort + UART_OFFSET_DLH, (dwBaudRateDivisor >> 8) & 0xFF);

    switch (lpszModeString[0])
    {
        case '5':
            bSettings |= (UART_LCR_WL_5 << UART_LCR_WL);
            break;
        case '6':
            bSettings |= (UART_LCR_WL_6 << UART_LCR_WL);
            break;
        case '7':
            bSettings |= (UART_LCR_WL_7 << UART_LCR_WL);
            break;
        case '8':
            bSettings |= (UART_LCR_WL_8 << UART_LCR_WL);
            break;
        default:
            return FALSE;
    }

    switch (lpszModeString[1])
    {
        case 'n':
        case 'N':
            bSettings |= (UART_LCR_PC_NONE << UART_LCR_PC);
            break;
        case 'o':
        case 'O':
            bSettings |= (UART_LCR_PC_ODD << UART_LCR_PC);
            break;
        case 'e':
        case 'E':
            bSettings |= (UART_LCR_PC_EVEN << UART_LCR_PC);
            break;
        default:
            return FALSE;
    }

    switch (lpszModeString[2])
    {
        case '1':
            bSettings |= (UART_LCR_STB_ONE << UART_LCR_STB);
            break;
        case '2':
            bSettings |= (UART_LCR_STB_TWO << UART_LCR_STB);
            break;
        default:
            return FALSE;
    }

    /* Write out settings */
    OUTB(hPort + UART_OFFSET_LCR, bSettings);

    return TRUE;
}

void EscapeHandler(void)
{
    static int iKeyStrokes;
    int iStrokesLeft;

    iKeyStrokes++;
    iStrokesLeft = ESC_STROKES_TO_QUIT - iKeyStrokes;

    if (iStrokesLeft > 0)
    {
        printf("Hit me %i time%s more to quit!\n",
               iStrokesLeft,
               iStrokesLeft > 1 ? "s" : "");
    }
    else
    {
        exit(0);
    }
}

/* Read a byte */
BYTE ReadByte(HPORT hPort)
{
    BYTE bStatus;

    do
    {
        if (kbhit() && (getch() == ESC_KEYCODE))
            EscapeHandler();

        /* Read the status register */
        bStatus = INB(hPort + UART_OFFSET_LSR);
    }
    /* Loop until data becomes ready */
    while ((bStatus & (1 << UART_LSR_DR)) == 0);

    /* Read the byte that is ready */
    return INB(hPort + UART_OFFSET_DATA);
}

/* Write a byte */
void WriteByte(HPORT hPort, BYTE bByte)
{
    BYTE bStatus;

    do
    {
        /* Read the status register */
        bStatus = INB(hPort + UART_OFFSET_LSR);
    }
    /* Loop until transmission hold register is empty */
    while ((bStatus & (1 << UART_LSR_THE)) == 0);

    /* Write the byte into data register */
    OUTB(hPort + UART_OFFSET_DATA, bByte);
}

/* Dummy function */
void CloseComPort(HPORT hPort)
{
}
